import abc
import typing


class CodecBackendInterface(metaclass=abc.ABCMeta):
    @classmethod
    def __subclasshook__(cls, __subclass: type) -> bool:
        return (
            hasattr(__subclass, "decode")
            and callable(__subclass.decode)
            and hasattr(__subclass, "encode")
            and callable(__subclass.encode)
            or NotImplemented
        )

    @abc.abstractmethod
    def decode(self, value: str) -> typing.Any:
        raise NotImplementedError

    @abc.abstractmethod
    def encode(self, value: dict[str, typing.Any]) -> str:
        raise NotImplementedError



import typing

import jwt
from jwt.exceptions import InvalidTokenError

from starlette.datastructures import Secret

from starlette_session.middleware.codecbackends.errors import DecodeError
from starlette_session.middleware.codecbackends.base import CodecBackendInterface


class JwtBackend(CodecBackendInterface):
    def __init__(
        self,
        key: typing.Union[str, Secret],
        algorithm: str,
    ):
        self.key = key
        self.algorithm = algorithm

    def decode(self, value: str) -> typing.Any:
        try:
            return jwt.decode(value, key=self.key, algorithms=[self.algorithm])
        except InvalidTokenError as ex:
            raise DecodeError("Failed to decode value") from ex

    def encode(self, data: dict[str, typing.Any]) -> str:
        return str(jwt.encode(data, key=self.key, algorithm=self.algorithm))



import json
import typing
from base64 import b64decode, b64encode

import itsdangerous
from itsdangerous.exc import BadSignature

from starlette.datastructures import Secret

from starlette_session.middleware.codecbackends.errors import DecodeError
from starlette_session.middleware.codecbackends.base import CodecBackendInterface


class SignerBackend(CodecBackendInterface):
    def __init__(
        self,
        key: typing.Union[str, Secret],
        max_age: typing.Optional[int] = 14 * 24 * 60 * 60,
    ):
        self.signer = itsdangerous.TimestampSigner(str(key))
        self.max_age = max_age

    def decode(self, value: str) -> typing.Any:
        try:
            data = self.signer.unsign(value, max_age=self.max_age)
            return json.loads(b64decode(data))
        except BadSignature as ex:
            raise DecodeError("Failed to decode value") from ex

    def encode(self, data: dict[str, typing.Any]) -> str:
        base64_encoded_data = b64encode(json.dumps(data).encode("utf-8"))
        return str(self.signer.sign(base64_encoded_data).decode("utf-8"))
